import { vtkObject } from 'vtk.js/Sources/interfaces';
import { Vector3 } from 'vtk.js/Sources/types';

/**
 *
 */
export interface IConeInitialValues {
  angle?: number;
}

export interface vtkCone extends vtkObject {
  /**
   * Given the point x evaluate the cone equation.
   * @param {Vector3} x The point coordinate.
   */
  evaluateFunction(x: Vector3): number;

  /**
   * Given the point x evaluate the equation for the cone gradient.
   * @param {Vector3} x The point coordinate.
   */
  evaluateGradient(x: Vector3): number[];

  /**
   * Get the angle of the cone.
   */
  getAngle(): number;

  /**
   * Set the value representing the angle of the cone.
   * @param {Number} angle The angle of the cone.
   */
  setAngle(angle: number): boolean;
}

/**
 * Method used to decorate a given object (publicAPI+model) with vtkCone characteristics.
 *
 * @param publicAPI object on which methods will be bounds (public)
 * @param model object on which data structure will be bounds (protected)
 * @param {IConeInitialValues} [initialValues] (default: {})
 */
export function extend(
  publicAPI: object,
  model: object,
  initialValues?: IConeInitialValues
): void;

/**
 * Method used to create a new instance of vtkCone.
 * @param {IConeInitialValues} [initialValues] for pre-setting some of its content
 */
export function newInstance(initialValues?: IConeInitialValues): vtkCone;

/**
 * vtkCone computes the implicit function and/or gradient for a cone. vtkCone is
 * a concrete implementation of vtkImplicitFunction.
 */
export declare const vtkCone: {
  newInstance: typeof newInstance;
  extend: typeof extend;
};
export default vtkCone;
